@@ -23,8 +23,6 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
-from __future__ import absolute_import, print_function
-
 import apt_pkg
 import copy
 from hashlib import md5
@@ -35,6 +33,7 @@ import shutil
 import threading
 import atexit
 import tempfile
+import pathlib
 try:
   from string import maketrans
 except ImportError:
@@ -59,7 +58,7 @@ import aptsources.distro
 import softwareproperties
 
 from .AptAuth import AptAuth
-from aptsources.sourceslist import (SourcesList, SourceEntry)
+from aptsources.sourceslist import (SourcesList, SourceEntry, Deb822SourceEntry)
 from softwareproperties.shortcuthandler import InvalidShortcutException
 from softwareproperties.shortcuts import shortcut_handler
 
@@ -68,7 +67,7 @@ from gi.repository import Gio
 
 class SoftwareProperties(object):
 
-  # known (whitelisted) channels
+  # directory containing channels
   CHANNEL_PATH="/usr/share/app-install/channels/"
 
   # release upgrades policy
@@ -85,9 +84,9 @@ class SoftwareProperties(object):
     RELEASE_UPGRADES_NEVER  : 'never',
   }
 
-  def __init__(self, datadir=None, options=None, rootdir="/"):
-    """ Provides the core functionality to configure the used software 
-        repositories, the corresponding authentication keys and 
+  def __init__(self, datadir=None, options=None, rootdir="/", deb822=False):
+    """ Provides the core functionality to configure the used software
+        repositories, the corresponding authentication keys and
         update automation """
     self.popconfile = rootdir+"/etc/popularity-contest.conf"
 
@@ -101,9 +100,9 @@ class SoftwareProperties(object):
     self.options = options
     self.datadir = datadir
 
-    self.sourceslist = SourcesList()
+    self.sourceslist = SourcesList(deb822=deb822)
     self.distro = aptsources.distro.get_distro()
-    
+
     self.seen_server = []
     self.modified_sourceslist = False
 
@@ -147,8 +146,8 @@ class SoftwareProperties(object):
     self.write_config()
 
   def get_update_automation_level(self):
-    """ Parse the apt cron configuration. Try to fit a predefined use case 
-        and return it. Special case: if the user made a custom 
+    """ Parse the apt cron configuration. Try to fit a predefined use case
+        and return it. Special case: if the user made a custom
         configurtation, that we cannot represent it will return None """
     if apt_pkg.config.find_i(softwareproperties.CONF_MAP["autoupdate"]) > 0:
         # Autodownload
@@ -170,9 +169,9 @@ class SoftwareProperties(object):
         return None
 
   def set_update_automation_level(self, state):
-    """ Set the apt periodic configurtation to the selected 
-        update automation level. To synchronize the cache update and the 
-        actual upgrading function, the upgrade function, e.g. unattended, 
+    """ Set the apt periodic configurtation to the selected
+        update automation level. To synchronize the cache update and the
+        actual upgrading function, the upgrade function, e.g. unattended,
         will run every day, if enabled. """
     if state == softwareproperties.UPDATE_INST_SEC:
         apt_pkg.config.set(softwareproperties.CONF_MAP["unattended"], str(1))
@@ -244,8 +243,8 @@ class SoftwareProperties(object):
     return True
 
   def get_popcon_participation(self):
-    """ Will return True if the user wants to participate in the popularity 
-        contest. Otherwise it will return False. Special case: if no 
+    """ Will return True if the user wants to participate in the popularity
+        contest. Otherwise it will return False. Special case: if no
         popcon is installed it will return False """
     if os.path.exists(self.popconfile):
         with open(self.popconfile) as f:
@@ -285,10 +284,10 @@ class SoftwareProperties(object):
         f.writelines(lines)
 
   def get_source_code_state(self):
-    """Return True if all distro componets are also available as 
+    """Return True if all distro componets are also available as
        source code. Otherwise return Flase. Special case: If the
        configuration cannot be represented return None"""
-  
+
     if len(self.distro.source_code_sources) < 1:
         # we don't have any source code sources, so
         # uncheck the button
@@ -337,7 +336,7 @@ class SoftwareProperties(object):
     print("\n")
 
   def massive_debug_output(self):
-    """Print the complete sources.list""" 
+    """Print the complete sources.list"""
     print("START SOURCES.LIST:")
     for source in self.sourceslist:
         print(source.str())
@@ -352,12 +351,12 @@ class SoftwareProperties(object):
 
   def enable_component(self, comp):
     """Enable a component of the distro"""
-    self.distro.enable_component(comp) 
+    self.distro.enable_component(comp)
     self.set_modified_sourceslist()
 
   def disable_component(self, comp):
     """Disable a component of the distro"""
-    self.distro.disable_component(comp) 
+    self.distro.disable_component(comp)
     self.set_modified_sourceslist()
 
   def _find_template_from_string(self, name):
@@ -401,7 +400,7 @@ class SoftwareProperties(object):
     for source in self.distro.source_code_sources:
         self.sourceslist.remove(source)
     self.set_modified_sourceslist()
-  
+
   def enable_source_code_sources(self):
     """Enable source code source for all distro sources"""
     sources = []
@@ -418,25 +417,32 @@ class SoftwareProperties(object):
                              source.dist,
                              source.comps,
                              "Added by software-properties",
-                             self.sourceslist.list.index(source)+1,
-                             source.file)
+                             self.sourceslist.index(source)+1,
+                             source.file,
+                             parent=source)
     for source in self.distro.cdrom_sources:
         self.sourceslist.add("deb-src",
                              self.distro.source_template.base_uri,
                              self.distro.source_template.name,
                              source.comps,
                              "Added by software-properties",
-                             self.sourceslist.list.index(source)+1,
-                             source.file)
+                             self.sourceslist.index(source)+1,
+                             source.file,
+                             parent=source)
     self.set_modified_sourceslist()
 
   def backup_sourceslist(self):
     """Store a backup of the source.list in memory"""
     self.sourceslist_backup = []
     for source in self.sourceslist.list:
-        source_bkp = SourceEntry(line=source.line,file=source.file)
+        if isinstance(source, SourceEntry):
+            source_bkp = SourceEntry(line=source.line,file=source.file)
+        elif isinstance(source, Deb822SourceEntry):
+            source_bkp = Deb822SourceEntry(section=source.line, file=source.file)
+        else:
+            raise TypeError(f"Unknown source: {source}")
         self.sourceslist_backup.append(source_bkp)
-  
+
   def _find_source_from_string(self, line):
     # ensure that we have a current list, it might have been changed underneath
     # us
@@ -460,13 +466,16 @@ class SoftwareProperties(object):
     for (index, entry) in enumerate(self.sourceslist.list):
       if str(entry) == old_entry:
         file = self.sourceslist.list[index].file
-        self.sourceslist.list[index] = SourceEntry(new_entry, file)
+        if isinstance(entry, Deb822SourceEntry):
+            self.sourceslist.list[index] = Deb822SourceEntry(new_entry, file)
+        else:
+            self.sourceslist.list[index] = SourceEntry(new_entry, file)
         self.set_modified_sourceslist()
         return True
     return False
 
   def revert(self):
-    """Revert all settings to the state when software-properties 
+    """Revert all settings to the state when software-properties
        was launched"""
     #FIXME: GPG keys are still missing
     self.restore_apt_conf()
@@ -476,7 +485,12 @@ class SoftwareProperties(object):
     """Restore the source list from the startup of the dialog"""
     self.sourceslist.list = []
     for source in self.sourceslist_backup:
-        source_reset = SourceEntry(line=source.line,file=source.file)
+        if isinstance(source, SourceEntry):
+            source_reset = SourceEntry(line=source.line,file=source.file)
+        elif isinstance(source, Deb822SourceEntry):
+            source_reset = Deb822SourceEntry(section=source.line, file=source.file)
+        else:
+            raise TypeError(f"Unknown source: {source}")
         self.sourceslist.list.append(source_reset)
     self.save_sourceslist()
     self.reload_sourceslist()
@@ -497,8 +511,11 @@ class SoftwareProperties(object):
     """Render a nice output to show the source in a treeview"""
     if source.template == None:
         if source.comment:
-            contents = "<b>%s</b> %s" % (escape(source.comment).strip(),
-                                         source.dist)
+            comment = (
+                source.comment.replace("\n", " ").strip()[:80]
+                + "[…]" if len(source.comment) > 80 else ""
+            )
+            contents = "<b>%s</b> %s" % (escape(comment), source.dist)
             # Only show the components if there are more than one
             if len(source.comps) > 1:
                 for c in source.comps:
@@ -569,7 +586,7 @@ class SoftwareProperties(object):
   def get_cdrom_sources(self):
     """Return the list of CDROM based distro sources"""
     return self.distro.cdrom_sources
-      
+
   def get_comp_download_state(self, comp):
     """Return a tuple: the first value describes if a component is enabled
        in the Internet repositories. The second value describes if the
@@ -579,7 +596,7 @@ class SoftwareProperties(object):
 
   def get_comp_child_state(self, template):
     """Return a tuple: the first value describes if a component is enabled
-       in one of the child source that matcth the given template. 
+       in one of the child source that matcth the given template.
        The second value describes if the first value is inconsistent."""
     comps = []
     for child in self.distro.child_sources:
@@ -591,17 +608,17 @@ class SoftwareProperties(object):
         return (True, False)
     elif len(comps) > 0 and\
         len(self.distro.enabled_comps ^ set(comps)) != 0:
-        # A matching child source does exist but doesn't include all 
+        # A matching child source does exist but doesn't include all
         # enabled distro components
         return(False, True)
     else:
         # There is no corresponding child source at all
         return (False, False)
-  
+
   def reload_sourceslist(self):
     self.sourceslist.refresh()
     self.sourceslist_visible=[]
-    self.distro.get_sources(self.sourceslist)    
+    self.distro.get_sources(self.sourceslist)
 
   def write_config(self):
     """Write the current apt configuration to file"""
@@ -616,6 +633,9 @@ class SoftwareProperties(object):
         break
     else:
       print("No config found, creating one")
+      dirname = os.path.dirname(f)
+      if not os.path.exists(dirname):
+        pathlib.Path(dirname).mkdir(parents=True)
       with open(conffiles[0], "w") as f:
         f.write("")
 
@@ -650,12 +670,12 @@ class SoftwareProperties(object):
             f.write("APT::Periodic::%s \"%s\";\n" % (i, cnf.find_i(i)))
 
   def save_sourceslist(self):
-    """Backup the existing sources.list files and write the current 
+    """Backup the existing sources.list files and write the current
        configuration"""
     self.sourceslist.backup(".save")
     self.sourceslist.save()
 
-  def _is_line_in_whitelisted_channel(self, srcline):
+  def _get_channel_from_line(self, srcline):
     """
     helper that checks if a given line is in the source list
     return the channel name or None if not found
@@ -671,14 +691,13 @@ class SoftwareProperties(object):
               return os.path.splitext(os.path.basename(filename))[0]
     return None
 
-  def check_and_add_key_for_whitelisted_shortcut(self, shortcut):
+  def check_and_add_key_for_channel_shortcut(self, shortcut):
     """
     helper that adds the gpg key of the channel to the apt
-    keyring *if* the channel is in the whitelist
-    /usr/share/app-install/channels
+    keyring *if* the channel is in /usr/share/app-install/channels
     """
     srcline = shortcut.SourceEntry().line
-    channel = self._is_line_in_whitelisted_channel(srcline)
+    channel = self._get_channel_from_line(srcline)
     if channel:
       keyp = "%s/%s.key" % (self.CHANNEL_PATH, channel)
       self.add_key(keyp)
@@ -719,7 +738,7 @@ class SoftwareProperties(object):
   def add_source_from_shortcut(self, shortcut, enable_source_code=False):
     """
     Add a source with the given shortcut and add the signing key if the
-    site is in whitelist or the shortcut implementer adds it.
+    site is a known channel or the shortcut implementer adds it.
     """
 
     deb_line = shortcut.SourceEntry().line
@@ -731,7 +750,7 @@ class SoftwareProperties(object):
     new_debsrc_entry = SourceEntry(debsrc_line, file)
     if new_deb_entry.invalid or new_debsrc_entry.invalid:
       return False
-    if not self.check_and_add_key_for_whitelisted_shortcut(shortcut):
+    if not self.check_and_add_key_for_channel_shortcut(shortcut):
       shortcut.add_key()
     self.sourceslist.add(new_deb_entry.type,
                          new_deb_entry.uri,
@@ -783,7 +802,10 @@ class SoftwareProperties(object):
     # file are gone and the file is not saved. we work around that
     # here
     if source.file != apt_pkg.config.find_file("Dir::Etc::sourcelist"):
-      self.sourceslist.list.append(SourceEntry("", file=source.file))
+      if source.file.endswith(".sources"):
+        self.sourceslist.list.append(Deb822SourceEntry("", file=source.file))
+      else:
+        self.sourceslist.list.append(SourceEntry("", file=source.file))
     try:
       self.sourceslist.remove(source)
     except ValueError:
