@@ -24,8 +24,6 @@
 
 # FIXME: why does this pass kapps around?????
 
-from __future__ import absolute_import, print_function
-
 import apt_pkg
 import tempfile
 from gettext import gettext as _
@@ -720,8 +718,10 @@ class SoftwarePropertiesQt(SoftwareProperties):
     home = QDir.homePath()
     if "SUDO_USER" in os.environ:
         home = os.path.expanduser("~%s" % os.environ["SUDO_USER"])
-    url = KUrl.fromPath(home)
-    filename = KFileDialog.getOpenFileName(url, 'application/pgp-keys', self.userinterface, _("Import key"))
+    filename = QFileDialog.getOpenFileName(self.userinterface,
+                                           _("Import key"),
+                                           home,
+                                           _("GPG key file (*.gpg)"))[0]
     if filename:
       if not self.add_key(filename):
         title = _("Error importing selected file")
